@@ -56,9 +56,25 @@
       ))
 
 ;;; ************************************************************************
-;;; SMART SETTINGS
+;;; SMART SETTINGS FOR THE ACTION AND ASSIST KEYS AND SMART MENUS
 ;;; ************************************************************************
 
+;; The following setting allows direct selection of Helm completion
+;; entries with the Smart Mouse Keys.  Otherwise, by default, helm
+;; disables all mouse keys while completing.  This setting requires
+;; the 'global_mouse' branch of the Helm git development tree because
+;; it has many changes for Hyperbole mouse support.  Otherwise, the
+;; setting will do nothing.
+;; Find it at: "https://github.com/rswgnu/helm/tree/global_mouse".
+;;
+;; If you change the value of this setting (e.g. simply comment it out),
+;;  you must restart Emacs for it to take full effect.  Setting this to
+;; 't' or 'nil' will not provide Hyperbole support.
+(eval-after-load "helm" '(if (boundp 'helm--disable-mouse-mode)
+			     ;; Using Helm version heavily modified for
+			     ;; Hyperbole Smart Key use.
+			     (setq helm-allow-mouse 'global-mouse-bindings)))
+
 ;; The Smart Menu system is an attractive in-buffer menu system that
 ;; predates Emacs menu systems; it is included in InfoDock.
 (defvar hkey-always-display-menu nil
@@ -66,7 +82,7 @@
 If a Smart Menu is already displayed, perform another Action or Assist Key
 function.")
 
-(defcustom hmouse-middle-flag (and (boundp 'infodock-version) infodock-version)
+(defcustom hmouse-middle-flag (and (boundp 'infodock-version) infodock-version t)
   "*Under InfoDock or when t, additionally bind the middle mouse button as an
 Action Key."
   :type 'boolean
@@ -186,9 +202,12 @@ obtained search string."
        ;; appeal to you. See "hui-em-but.el" for how this works.
        (hproperty:cycle-but-color nil)
        ;;
+       ;; Non-nil means highlight all explict buttons with `hproperty:but-face'.
+       (setq hproperty:but-highlight-flag t)
+       ;;
        ;; Non-nil means visually emphasize that button under mouse cursor is
        ;; selectable.
-       (setq hproperty:but-emphasize-p nil)
+       (setq hproperty:but-emphasize-flag nil)
        ;;
        ;; If you find that the Hyperbole button flash time is too slow
        ;; or too fast, adjust it here.
@@ -208,6 +227,9 @@ obtained search string."
        ;; appeal to you. See "hui-xe-but.el" for how this works.
        (hproperty:cycle-but-color nil)
        ;;
+       ;; Non-nil means highlight all explict buttons with `hproperty:but-face'.
+       (setq hproperty:but-highlight-flag t)
+       ;;
        ;; Non-nil means visually emphasize that button under mouse cursor is
        ;; selectable.
        (setq hproperty:but-emphasize-p nil)
