@@ -1,33 +1,18 @@
-;;; hvm.el --- Support Hyperbole buttons in mail reader: Vm.
-
-;; Copyright (C) 1991-1995, BeOpen.com and the Free Software Foundation, Inc.
-;; Developed with support from Motorola Inc.
-
-;; Author: Bob Weiner, Brown U.
-;; Maintainer: Mats Lidell <matsl@contactor.se>
-;; Keywords: hypermedia, mail
-
+;;; hvm.el ---  GNU Hyperbole buttons in mail reader: Vm
+;;
+;; Author:       Bob Weiner
+;;
+;; Orig-Date:    10-Oct-91 at 01:51:12
+;;
+;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; See the "HY-COPY" file for license information.
+;;
 ;; This file is part of GNU Hyperbole.
 
-;; GNU Hyperbole is free software; you can redistribute it and/or
-;; modify it under the terms of the GNU General Public License as
-;; published by the Free Software Foundation; either version 3, or (at
-;; your option) any later version.
-
-;; GNU Hyperbole is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
-
-;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
-
 ;;; Commentary:
 ;;
 ;;   Automatically configured for use in "hyperbole.el".
-;;   If hsite loading fails prior to initializing Hyperbole Vm support,
+;;   If hsettings loading fails prior to initializing Hyperbole Vm support,
 ;;
 ;;       {M-x Vm-init RET}
 ;;
@@ -35,21 +20,30 @@
 ;;
 
 ;;; Code:
-
-;;;
+;;; ************************************************************************
 ;;; Other required Elisp libraries
-;;;
+;;; ************************************************************************
 
-(require 'hmail)
-(load "hsmail")
-(require 'vm)
-(or (and (fboundp 'vm-edit-message) (fboundp 'vm-edit-message-end))
-    (load "vm-edit"))
-(vm-session-initialization)
+;; VM is not available as a regular Emacs package, so it is somewhat
+;; difficult to install properly.  Requiring it below will therefore
+;; often trigger an error and prevent Hyperbole from properly
+;; loading, so prevent this error with a condition-case around this
+;; whole file.
 
-;;;
+(condition-case nil
+(progn
+
+(eval-and-compile
+  (require 'hmail)
+  (load "hsmail")
+  (when (require 'vm nil t)
+    (or (and (fboundp 'vm-edit-message) (fboundp 'vm-edit-message-end))
+	(load "vm-edit"))
+    (vm-session-initialization))) ;; this requires all of vm's modules
+
+;;; ************************************************************************
 ;;; Public variables
-;;;
+;;; ************************************************************************
 
 ;;; Current versions of VM define this next variable in "vm-vars.el".  We
 ;;; define it here for earlier VM versions.
@@ -60,6 +54,10 @@
 ;;; default `text-mode', is used for editing.
 (setq vm-edit-message-mode 'vm-edit-mode)
 
+;;; ************************************************************************
+;;; Public functions
+;;; ************************************************************************
+
 (defun vm-edit-mode ()
   "Major mode for editing vm mail messages.
   Special commands:\\{vm-edit-message-map}
@@ -75,10 +73,7 @@ if that value is non-nil."
 ;;  (run-hooks 'vm-edit-message-hook)
   )
 
-;;;
-;;; Public functions
-;;;
-
+;;;###autoload
 (defun Vm-init ()
   "Initializes Hyperbole support for Vm mail reading."
   (interactive)
@@ -99,7 +94,7 @@ if that value is non-nil."
   (defalias 'rmail:msg-forward   'vm-forward-message)
   (defalias 'rmail:summ-msg-to   'vm-follow-summary-cursor)
   (defalias 'rmail:summ-new      'vm-summarize)
-  (if (interactive-p)
+  (if (called-interactively-p 'interactive)
       (message "Hyperbole VM mail reader support initialized."))
   )
 
@@ -180,9 +175,9 @@ Returns t if successful, else nil or signals error."
   (message
    "(Vm-Summ-undelete-all: Vm doesn't have an undelete all msgs function."))
 
-;;;
+;;; ************************************************************************
 ;;; Private functions
-;;;
+;;; ************************************************************************
 
 (defun Vm-msg-end ()
   "Returns end point for current Vm message, including Hyperbole button data.
@@ -195,76 +190,37 @@ Has side-effect of widening buffer."
 	(match-beginning 0)
       (point-max))))
 
-;;; Overlay version of this function from "vm-page.el" to hide any
+;;; Redefine version of this function from "vm-page.el" to hide any
 ;;; Hyperbole button data whenever a message is displayed in its entirety.
 (defun vm-show-current-message ()
-  (and vm-display-using-mime
-       vm-auto-decode-mime-messages
-       (if vm-mail-buffer
-	   (not (vm-buffer-variable-value vm-mail-buffer 'vm-mime-decoded))
-	 (not vm-mime-decoded))
-       (not (vm-mime-plain-message-p (car vm-message-pointer)))
-       (condition-case data
-	   (vm-decode-mime-message)
-	 (vm-mime-error (vm-set-mime-layout-of (car vm-message-pointer)
-					       (car (cdr data)))
-			(message "%s" (car (cdr data))))))
-  (if (and (natnump vm-fill-paragraphs-containing-long-lines)
-	   (vm-mime-plain-message-p (car vm-message-pointer)))
-      (let ((needmsg (> (- (vm-text-end-of (car vm-message-pointer))
-			   (vm-text-of (car vm-message-pointer)))
-			12000)))
-	(if needmsg
-	    (message "Searching for paragraphs to fill..."))
-	(vm-fill-paragraphs-containing-long-lines
-	 vm-fill-paragraphs-containing-long-lines
-	 (vm-text-of (car vm-message-pointer))
-	 (vm-text-end-of (car vm-message-pointer)))
-	(if needmsg
-	    (message "Searching for paragraphs to fill... done"))))
-  (vm-save-buffer-excursion
-   (save-excursion
-     (save-excursion
-       (goto-char (point-min))
-       (widen)
-       ; (hmail:msg-narrow (point-min) (Vm-msg-end)))
-       ; (narrow-to-region (point) (vm-text-end-of (car vm-message-pointer))))
-       (hmail:msg-narrow (point) (vm-text-end-of (car vm-message-pointer))))
-     (and vm-honor-page-delimiters
-	  (save-excursion
-	    (if (search-forward page-delimiter nil t)
-		(progn
-		  (goto-char (match-beginning 0))
-		  (not (looking-at (regexp-quote hmail:hbdata-sep))))))
-	  (progn
-	    (if (looking-at page-delimiter)
-		(forward-page 1))
-	    (vm-narrow-to-page))))
-   ;; don't mark the message as read if the user can't see it!
-   (if (vm-get-buffer-window (current-buffer))
-       (progn
+  (save-excursion
+    (save-excursion
+      (goto-char (point-min))
+      (hmail:msg-narrow (point-min) (Vm-msg-end)))
+    (and vm-honor-page-delimiters
 	 (save-excursion
-	   (setq vm-system-state 'showing)
-	   (if vm-mail-buffer
-	       (vm-set-buffer-variable vm-mail-buffer 'vm-system-state
-				       'showing))
-	   ;; We could be in the presentation buffer here.  Since
-	   ;; the presentation buffer's message pointer and sole
-	   ;; message are a mockup, they will cause trouble if
-	   ;; passed into the undo/update system.  So we switch
-	   ;; into the real message buffer to do attribute
-	   ;; updates.
-	   (vm-select-folder-buffer)
-	   (cond ((vm-new-flag (car vm-message-pointer))
-		  (vm-set-new-flag (car vm-message-pointer) nil)))
-	   (cond ((vm-unread-flag (car vm-message-pointer))
-		  (vm-set-unread-flag (car vm-message-pointer) nil))))
-	 (vm-update-summary-and-mode-line)
-	 (vm-howl-if-eom))
-     (if (fboundp 'hproperty:but-create) (hproperty:but-create))
-     (vm-update-summary-and-mode-line))))
+	   (if (search-forward page-delimiter nil t)
+	       (progn
+		 (goto-char (match-beginning 0))
+		 (not (looking-at (regexp-quote hmail:hbdata-sep))))))
+	 (progn
+	   (if (looking-at page-delimiter)
+	       (forward-page 1))
+	   (vm-narrow-to-page))))
+  ;; don't mark the message as read if the user can't see it!
+  (if (vm-get-buffer-window (current-buffer))
+      (progn
+	(setq vm-system-state 'showing)
+	(cond ((vm-new-flag (car vm-message-pointer))
+	       (vm-set-new-flag (car vm-message-pointer) nil)))
+	(cond ((vm-unread-flag (car vm-message-pointer))
+	       (vm-set-unread-flag (car vm-message-pointer) nil)))
+	(vm-update-summary-and-mode-line)
+	(vm-howl-if-eom))
+    (if (fboundp 'hproperty:but-create) (hproperty:but-create))
+    (vm-update-summary-and-mode-line)))
 
-;;; Overlay version of this function from "vm-page.el" to treat end of
+;;; Redefine version of this function from "vm-page.el" to treat end of
 ;;; text (excluding Hyperbole button data) as end of message.
 (defun vm-scroll-forward-internal (arg)
   (let ((direction (prefix-numeric-value arg))
@@ -302,7 +258,7 @@ Has side-effect of widening buffer."
 	       (set-window-point w (point))
 	       'end-of-message)))))))
 
-;;; Overlay version of this function from "vm-page.el" (called by
+;;; Redefine version of this function from "vm-page.el" (called by
 ;;; vm-scroll-* functions).  Make it keep Hyperbole button data hidden.
 (defun vm-widen-page ()
   (if (or (> (point-min) (vm-text-of (car vm-message-pointer)))
@@ -313,11 +269,11 @@ Has side-effect of widening buffer."
 			    (vm-text-of (car vm-message-pointer))
 			  (vm-text-end-of (car vm-message-pointer))))))
 
-;;; Overlay version of this function from "vm-edit.el" to hide
+;;; Redefine version of this function from "vm-edit.el" to hide
 ;;; Hyperbole button data when insert edited message from temporary buffer.
 (hypb:function-overload 'vm-edit-message nil '(hmail:msg-narrow))
 
-;;; Overlay version of this function from "vm-edit.el" to hide
+;;; Redefine version of this function from "vm-edit.el" to hide
 ;;; Hyperbole button data when insert edited message from temporary buffer.
 (defun vm-edit-message-end ()
   "End the edit of a message and copy the result to its folder."
@@ -358,22 +314,21 @@ Has side-effect of widening buffer."
       (if (buffer-modified-p)
 	  (progn
 	    (widen)
-	    (save-excursion
-	      (set-buffer (vm-buffer-of (vm-real-message-of (car mp))))
+	    (with-current-buffer (vm-buffer-of (vm-real-message-of (car mp)))
 	      (if (not (memq (vm-real-message-of (car mp)) vm-message-list))
 		  (error "The original copy of this message has been expunged."))
 	      (vm-save-restriction
 	       (widen)
 	       (goto-char (vm-headers-of (vm-real-message-of (car mp))))
 	       (let ((vm-message-pointer mp)
-                     opoint
+		     opoint
 		     (buffer-read-only nil))
 		 (setq opoint (point))
 		 (insert-buffer-substring edit-buf)
 		 (delete-region
 		  (point) (vm-text-end-of (vm-real-message-of (car mp))))
-		 (vm-discard-cached-data))
-	       (hmail:msg-narrow)
+		 (vm-discard-cached-data)
+		 (hmail:msg-narrow))
 	       (vm-set-edited-flag-of (car mp) t)
 	       (vm-set-edit-buffer-of (car mp) nil))
 	      (set-buffer (vm-buffer-of (car mp)))
@@ -413,6 +368,7 @@ Has side-effect of widening buffer."
       (set-buffer-modified-p nil)
       (kill-buffer edit-buf))))
 
+
 ;;; Define this function if the VM version in use doesn't have it.
 (or (fboundp 'vm-goto-message-at-point)
 (defun vm-goto-message-at-point ()
@@ -440,9 +396,9 @@ Has side-effect of widening buffer."
 ;;; See "vm-reply.el".
 (var:append 'vm-mail-mode-hook '(hmail:msg-narrow))
 
-;;; Overlay this function from "vm-folder.el" called whenever new mail is
+;;; Redefine this function from "vm-folder.el" called whenever new mail is
 ;;; incorporated so that it will highlight Hyperbole buttons when possible.
-;;  Returns non-nil if there were any new messages.
+;;  Returns list of new messages if there were any new messages, nil otherwise.
 (defun vm-assimilate-new-messages (&optional
 				   dont-read-attributes
 				   gobble-order
@@ -457,7 +413,8 @@ Has side-effect of widening buffer."
        (vm-build-message-list)
        (if (or (null tail-cons) (cdr tail-cons))
 	   (progn
-	     (setq vm-ml-sort-keys nil)
+             (if (not vm-assimilate-new-messages-sorted)
+                 (setq vm-ml-sort-keys nil))
 	     (if dont-read-attributes
 		 (vm-set-default-attributes (cdr tail-cons))
 	       (vm-read-attributes (cdr tail-cons)))
@@ -494,7 +451,8 @@ Has side-effect of widening buffer."
     ;; not be mangled.
     (setq new-messages (copy-sequence new-messages))
     ;; add the labels
-    (if (and new-messages labels vm-burst-digest-messages-inherit-labels)
+    (if (and new-messages labels (boundp 'vm-burst-digest-messages-inherit-labels)
+	     vm-burst-digest-messages-inherit-labels)
 	(let ((mp new-messages))
 	  (while mp
 	    (vm-set-labels-of (car mp) (copy-sequence labels))
@@ -506,11 +464,11 @@ Has side-effect of widening buffer."
 	  ;; changes it needs to make.
 	  (vm-update-summary-and-mode-line)
 	  (vm-sort-messages "thread")))
-    (if (and new-messages
+    (if (and new-message
 	     (or vm-arrived-message-hook vm-arrived-messages-hook)
 	     ;; Run the hooks only if this is not the first
 	     ;; time vm-assimilate-new-messages has been called
-	     ;; in this folder. 
+	     ;; in this folder.
 	     (not first-time))
 	(let ((new-messages new-messages))
 	  ;; seems wise to do this so that if the user runs VM
@@ -536,7 +494,8 @@ Has side-effect of widening buffer."
 		  (vm-build-virtual-message-list new-messages)
 		  (if (or (null tail-cons) (cdr tail-cons))
 		      (progn
-			(setq vm-ml-sort-keys nil)
+                        (if (not vm-assimilate-new-messages-sorted)
+                            (setq vm-ml-sort-keys nil))
 			(if (vectorp vm-thread-obarray)
 			    (vm-build-threads (cdr tail-cons)))
 			(vm-set-summary-redo-start-point
@@ -553,29 +512,35 @@ Has side-effect of widening buffer."
 			      (vm-update-summary-and-mode-line)
 			      (vm-sort-messages "thread")))))))
 	    (setq b-list (cdr b-list)))))
+    (if (and new-messages vm-ml-sort-keys)
+        (vm-sort-messages vm-ml-sort-keys))
     new-messages ))
 
-;;; Overlay version of `vm-force-mode-line-update' from "vm-folder.el"
+;;; Redefine version of `vm-force-mode-line-update' from "vm-folder.el"
 ;;; to highlight Hyperbole buttons in summary buffers.
 (defun vm-force-mode-line-update ()
   "Force a mode line update in all frames."
   (if vm-summary-buffer
-      (save-excursion
-	(set-buffer vm-summary-buffer)
+      (with-current-buffer vm-summary-buffer
 	(if (fboundp 'hproperty:but-create) (hproperty:but-create))))
   (if (fboundp 'force-mode-line-update)
       (force-mode-line-update t)
-    (save-excursion
-      (set-buffer (other-buffer))
+    (with-current-buffer (other-buffer)
       (set-buffer-modified-p (buffer-modified-p)))))
 
-;;;
+;;; ************************************************************************
 ;;; Private variables
-;;;
+;;; ************************************************************************
 
 (defvar Vm-msg-start-regexp "\n\nFrom \\|\n\001\001\001\001"
   "Regular expression that begins a Vm mail message.")
 
 (provide 'hvm)
+)
+(error nil))
+
+;; Local Variables:
+;; no-byte-compile: t
+;; End:
 
 ;;; hvm.el ends here
